import CRUDBase from '../CRUDBase.js'

class ClientResource extends CRUDBase {
    endpoint = '/api/v1/ldap/clients'
    required_fields = ['name', 'uid', 'password']

    item = 'LDAP Client'
    plural = 'LDAP Clients'

    listing_definition = {
        columns: [
            {
                name: 'Client Name',
                field: 'name',
            },
            {
                name: 'User ID',
                field: 'uid',
            },
        ],
        actions: [
            {
                type: 'resource',
                position: 'main',
                action: 'insert',
                text: 'Create New',
                color: 'success',
            },
            {
                type: 'resource',
                position: 'row',
                action: 'update',
                icon: 'fa fa-edit',
                color: 'primary',
            },
        ],
    }

    form_definition = {
        fields: [
            {
                name: 'Provider Name',
                field: 'name',
                placeholder: 'Awesome External App',
                required: true,
                type: 'text',
            },
            {
                name: 'User ID',
                field: 'uid',
                placeholder: 'some_username',
                required: true,
                type: 'text',
            },
            {
                name: 'Password',
                field: 'password',
                required: ['insert'],
                type: 'password',
            },
        ],
    }
}

const ldap_client = new ClientResource()
export { ldap_client }
